==> karlender-v0.6.0.tar.gz: karlender-v0.6.0/src/widgets/dumb/event_tag.rs <==
use gdk4::subclass::prelude::ObjectSubclassIsExt;
use std::cell::Cell;

use crate::domain::KarlenderColor;
use gtk::{prelude::*, GestureClick};

#[widget(extends gtk::Box)]
#[template(file = "event_tag.ui")]
pub struct EventTag {
    #[property_string]
    pub label: Cell<String>,
    #[property_u64]
    pub num: Cell<u64>,
    #[property_bool]
    pub small: Cell<bool>,
    #[property_bool]
    pub transparent: Cell<bool>,

    #[template_child]
    pub event_label_scrolled_window: TemplateChild<gtk::ScrolledWindow>,
    #[template_child]
    pub event_tag_label: TemplateChild<gtk::Label>,
    #[template_child]
    pub events_placeholder_num: TemplateChild<gtk::Label>,
    #[template_child]
    pub events_placeholder_label: TemplateChild<gtk::Label>,

    #[signal]
    click: (),
}

impl EventTag {
    pub fn constructed(&self) {
        let _self = self;
        let gesture_click = GestureClick::builder().build();
        gesture_click.connect_released(glib::clone!(@weak _self => move |_, _, _, _| {
            _self.emit_click();
        }));
        self.add_controller(&gesture_click);
    }

    pub fn new(label: String, num: u64, color: KarlenderColor, small: bool) -> Self {
        let s: Self = glib::Object::new(&[("label", &label), ("num", &num), ("small", &small)])
            .expect("Failed to create EventTag");
        s.add_css_class(color.style_class());
        if small {
            s.add_css_class("font-small");
        }
        s
    }

    pub fn set_folded(&self, folded: bool) {
        // self.event_tag_label().set_visible(!folded);
        self.event_label_scrolled_window().set_visible(!folded);
        self.events_placeholder_num().set_visible(folded);
        self.events_placeholder_label().set_visible(folded);
    }

    pub fn label(&self) -> String {
        self.property("label")
    }

    pub fn connect_click(&self, c: impl Fn(&Self) + 'static) {
        self.auto_cleanup_signal_handler(self._connect_click(c));
    }
}

==> karlender-v0.9.1.tar.gz: karlender-v0.9.1/src/widgets/dumb/event_tag.rs <==
use gdk4::subclass::prelude::ObjectSubclassIsExt;
use gtk::{ffi::GTK_STYLE_PROVIDER_PRIORITY_APPLICATION, prelude::*, CssProvider, GestureClick};
use std::cell::Cell;

use crate::domain::get_text_color;

#[widget(extends gtk::Box)]
#[template(file = "event_tag.ui")]
pub struct EventTag {
    #[property_string]
    pub label: Cell<String>,
    #[property_u64]
    pub num: Cell<u64>,
    #[property_bool]
    pub small: Cell<bool>,
    #[property_bool]
    pub transparent: Cell<bool>,

    #[template_child]
    pub color_box: TemplateChild<gtk::Box>,
    #[template_child]
    pub event_label_scrolled_window: TemplateChild<gtk::ScrolledWindow>,
    #[template_child]
    pub event_tag_label: TemplateChild<gtk::Label>,
    #[template_child]
    pub events_placeholder_num: TemplateChild<gtk::Label>,
    #[template_child]
    pub events_placeholder_label: TemplateChild<gtk::Label>,

    #[signal]
    click: (),
}

impl EventTag {
    pub fn constructed(&self) {
        let _self = self;
        let gesture_click = GestureClick::builder().build();
        gesture_click.connect_released(glib::clone!(@weak _self => move |_, _, _, _| {
            _self.emit_click();
        }));
        self.add_controller(&gesture_click);
    }

    pub fn new(label: String, num: u64, color: &str, small: bool) -> Self {
        let s: Self = glib::Object::new(&[("label", &label), ("num", &num), ("small", &small)])
            .expect("Failed to create EventTag");
        let color = color.to_string();
        s.connect_realize(move |s| s.realized(&color));
        if small {
            s.add_css_class("font-small");
        }
        s
    }

    fn realized(&self, color: &str) {
        let css = CssProvider::new();
        css.load_from_data(
            format!(
                "* {{ background-color: {}; color: {};}} ",
                color,
                get_text_color(color)
            )
            .as_bytes(),
        );
        self.color_box()
            .style_context()
            .add_provider(&css, GTK_STYLE_PROVIDER_PRIORITY_APPLICATION as u32 + 1);
    }

    pub fn set_folded(&self, folded: bool) {
        self.event_label_scrolled_window().set_visible(!folded);
        self.events_placeholder_num().set_visible(folded);
        self.events_placeholder_label().set_visible(folded);
    }

    pub fn label(&self) -> String {
        self.property("label")
    }

    pub fn connect_click(&self, c: impl Fn(&Self) + 'static) {
        self.auto_cleanup_signal_handler(self._connect_click(c));
    }
}
